"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Executor_1 = require("./Executor");
var util_1 = require("../common/util");
var lang_1 = require("@dojo/core/lang");
var Formatter_1 = require("../browser/Formatter");
var Task_1 = require("@dojo/core/async/Task");
var Browser = (function (_super) {
    __extends(Browser, _super);
    function Browser(config) {
        var _this = _super.call(this, {
            basePath: '/',
            browserSuites: []
        }) || this;
        if (config) {
            _this.configure(config);
        }
        _this._formatter = new Formatter_1.default(_this.config);
        window.addEventListener('unhandledRejection', function (event) {
            console.warn('Unhandled rejection:', event);
            _this.emit('error', event.reason);
        });
        window.addEventListener('error', function (event) {
            console.warn('Unhandled error:', event);
            var error = new Error(event.message);
            error.stack = event.filename + ":" + event.lineno + ":" + event.colno;
            _this.emit('error', error);
        });
        return _this;
    }
    Browser.initialize = function (config) {
        return Executor_1.initialize(Browser, config);
    };
    Object.defineProperty(Browser.prototype, "environment", {
        get: function () {
            return 'browser';
        },
        enumerable: true,
        configurable: true
    });
    Browser.prototype.loadScript = function (script) {
        var _this = this;
        if (script == null) {
            return Task_1.default.resolve();
        }
        if (typeof script === 'string') {
            script = [script];
        }
        return script.reduce(function (previous, script) {
            if (script[0] !== '/') {
                script = "" + _this.config.basePath + script;
            }
            return previous.then(function () { return injectScript(script); });
        }, Task_1.default.resolve());
    };
    Browser.prototype._loadSuites = function () {
        var config = lang_1.duplicate(this.config);
        config.suites = config.suites.concat(config.browserSuites);
        if (config.browserLoader) {
            config.loader = config.browserLoader;
        }
        return _super.prototype._loadSuites.call(this, config);
    };
    Browser.prototype._processOption = function (name, value) {
        switch (name) {
            case 'basePath':
                this.config[name] = util_1.parseValue(name, value, 'string');
                break;
            case 'browserSuites':
                this.config[name] = util_1.parseValue(name, value, 'string[]');
                break;
            default:
                _super.prototype._processOption.call(this, name, value);
                break;
        }
    };
    Browser.prototype._resolveConfig = function () {
        var _this = this;
        return _super.prototype._resolveConfig.call(this).then(function () {
            var config = _this.config;
            if (!config.internPath) {
                config.internPath = 'node_modules/intern/';
            }
            ['suites', 'browserSuites'].forEach(function (name) {
                config[name] = config[name].filter(function (suite) {
                    if (/[*?]/.test(suite)) {
                        console.warn("Globs may not be used for browser suites: \"" + suite + "\"");
                        return false;
                    }
                    return true;
                });
            });
            ['basePath', 'internPath'].forEach(function (key) {
                config[key] = util_1.normalizePathEnding(config[key]);
            });
            if (config.internPath[0] !== '/') {
                config.internPath = "" + config.basePath + config.internPath;
            }
        });
    };
    return Browser;
}(Executor_1.default));
exports.default = Browser;
function injectScript(path) {
    return new Task_1.default(function (resolve, reject) {
        var scriptTag = document.createElement('script');
        scriptTag.addEventListener('load', function () {
            resolve();
        });
        scriptTag.addEventListener('error', function (event) {
            console.error("Error loading " + path + ":", event);
            reject(new Error("Unable to load " + path));
        });
        scriptTag.src = path;
        document.body.appendChild(scriptTag);
    });
}
//# sourceMappingURL=Browser.js.map