"use strict";
var __extends = (this && this.__extends) || (function () {
    var extendStatics = Object.setPrototypeOf ||
        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
        function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };
    return function (d, b) {
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    };
})();
Object.defineProperty(exports, "__esModule", { value: true });
var Executor_1 = require("./Executor");
var util_1 = require("../common/util");
var util_2 = require("../node/util");
var lang_1 = require("@dojo/core/lang");
var Formatter_1 = require("../node/Formatter");
var path_1 = require("path");
var istanbul_lib_hook_1 = require("istanbul-lib-hook");
var Pretty_1 = require("../reporters/Pretty");
var Simple_1 = require("../reporters/Simple");
var Benchmark_1 = require("../reporters/Benchmark");
var Promise_1 = require("@dojo/shim/Promise");
var istanbul_lib_instrument_1 = require("istanbul-lib-instrument");
var istanbul_lib_source_maps_1 = require("istanbul-lib-source-maps");
var Node = (function (_super) {
    __extends(Node, _super);
    function Node(config) {
        var _this = _super.call(this, {
            basePath: process.cwd() + path_1.sep,
            nodeSuites: []
        }) || this;
        if (config) {
            _this.configure(config);
        }
        _this.registerReporter('pretty', Pretty_1.default);
        _this.registerReporter('simple', Simple_1.default);
        _this.registerReporter('benchmark', Benchmark_1.default);
        _this._formatter = new Formatter_1.default(_this.config);
        process.on('unhandledRejection', function (reason, promise) {
            console.warn('Unhandled rejection:', promise);
            _this.emit('error', reason);
        });
        process.on('uncaughtException', function (reason) {
            console.warn('Unhandled error');
            _this.emit('error', reason);
        });
        return _this;
    }
    Node.initialize = function (config) {
        return Executor_1.initialize(Node, config);
    };
    Object.defineProperty(Node.prototype, "environment", {
        get: function () {
            return 'node';
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(Node.prototype, "sourceMapStore", {
        get: function () {
            return this._sourceMaps;
        },
        enumerable: true,
        configurable: true
    });
    Node.prototype.instrumentCode = function (code, filename) {
        this.log('Instrumenting', filename);
        var sourceMap = util_2.readSourceMap(filename, code);
        this._sourceMaps.registerMap(filename, sourceMap);
        return this._instrumenter.instrumentSync(code, path_1.normalize(filename), sourceMap);
    };
    Node.prototype.loadScript = function (script) {
        return util_2.loadScript(script);
    };
    Node.prototype.shouldInstrumentFile = function (filename) {
        var excludeInstrumentation = this.config.excludeInstrumentation;
        if (excludeInstrumentation === true) {
            return false;
        }
        var basePath = this._instrumentBasePath;
        filename = util_2.normalizePath(filename);
        return filename.indexOf(basePath) === 0 && !excludeInstrumentation.test(filename.slice(basePath.length));
    };
    Node.prototype._beforeRun = function () {
        var _this = this;
        return _super.prototype._beforeRun.call(this).then(function () {
            var config = _this.config;
            _this._instrumenter = istanbul_lib_instrument_1.createInstrumenter(lang_1.mixin({}, config.instrumenterOptions, {
                preserveComments: true,
                produceSourceMap: true
            }));
            _this._sourceMaps = istanbul_lib_source_maps_1.createSourceMapStore();
            if (_this.config.excludeInstrumentation !== true) {
                _this._setInstrumentationHooks();
            }
            var suite = _this._rootSuite;
            suite.grep = config.grep;
            suite.timeout = config.defaultTimeout;
            suite.bail = config.bail;
        });
    };
    Node.prototype._loadSuites = function (config) {
        config = lang_1.duplicate(config || this.config);
        config.suites = config.suites.concat(config.nodeSuites);
        if (config.nodeLoader) {
            config.loader = config.nodeLoader;
        }
        return _super.prototype._loadSuites.call(this, config);
    };
    Node.prototype._processOption = function (name, value) {
        switch (name) {
            case 'nodeSuites':
                this.config[name] = util_1.parseValue(name, value, 'string[]');
                break;
            default:
                _super.prototype._processOption.call(this, name, value);
                break;
        }
    };
    Node.prototype._resolveConfig = function () {
        var _this = this;
        return _super.prototype._resolveConfig.call(this).then(function () {
            var config = _this.config;
            if (!config.internPath) {
                config.internPath = path_1.dirname(path_1.dirname(__dirname));
            }
            config.internPath = "" + path_1.relative(process.cwd(), config.internPath) + path_1.sep;
            if (config.reporters.length === 0) {
                config.reporters = [{ reporter: 'simple' }];
            }
            if (!config.nodeSuites) {
                config.nodeSuites = [];
            }
            if (config.benchmarkConfig) {
                config.reporters.push({
                    reporter: 'benchmark',
                    options: config.benchmarkConfig
                });
            }
            _this._instrumentBasePath = util_2.normalizePath("" + path_1.resolve(config.basePath || '') + path_1.sep);
            return Promise_1.default.all(['suites', 'nodeSuites'].map(function (property) {
                return util_2.expandFiles(config[property]).then(function (expanded) {
                    config[property] = expanded;
                });
            })).then(function () { });
        });
    };
    Node.prototype._setInstrumentationHooks = function () {
        var _this = this;
        istanbul_lib_hook_1.hookRunInThisContext(function (filename) { return _this.shouldInstrumentFile(filename); }, function (code, filename) { return _this.instrumentCode(code, filename); });
        this._unhookRequire = istanbul_lib_hook_1.hookRequire(function (filename) { return _this.shouldInstrumentFile(filename); }, function (code, filename) { return _this.instrumentCode(code, filename); });
    };
    Node.prototype._removeInstrumentationHooks = function () {
        istanbul_lib_hook_1.unhookRunInThisContext();
        if (this._unhookRequire) {
            this._unhookRequire();
            this._unhookRequire = null;
        }
    };
    return Node;
}(Executor_1.default));
exports.default = Node;
//# sourceMappingURL=Node.js.map