import Node, { Config as BaseConfig, Events as BaseEvents } from './Node';
import Tunnel, { TunnelOptions, DownloadProgressEvent } from 'digdug/Tunnel';
import { BrowserStackOptions } from 'digdug/BrowserStackTunnel';
import { SeleniumOptions } from 'digdug/SeleniumTunnel';
import Server from '../Server';
import Task from '@dojo/core/async/Task';
import Suite from '../Suite';
import Environment from '../Environment';
import Command from 'leadfoot/Command';
export default class WebDriver extends Node<Events, Config> {
    static initialize(config?: Partial<Config>): WebDriver;
    server: Server;
    tunnel: Tunnel;
    protected _rootSuites: Suite[];
    protected _tunnels: {
        [name: string]: typeof Tunnel;
    };
    constructor(config?: Partial<Config>);
    readonly environment: string;
    addSuite(adder: (parentSuite: Suite) => void): void;
    registerTunnel(name: string, Class: typeof Tunnel): void;
    protected _afterRun(): Task<void>;
    protected _beforeRun(): Task<any>;
    protected _createServer(): Server;
    protected _createSessionSuites(): Task<void> | undefined;
    protected _processOption(name: keyof Config, value: any): void;
    protected _loadSuites(): Task<void>;
    protected _resolveConfig(): Task<void>;
    protected _runTests(): Task<any>;
}
export interface Config extends BaseConfig {
    capabilities: {
        name?: string;
        build?: string;
        [key: string]: any;
    };
    connectTimeout: number;
    environments: EnvironmentSpec[];
    leaveRemoteOpen: boolean | 'fail';
    maxConcurrency: number;
    serveOnly: boolean;
    serverPort: number;
    serverUrl: string;
    runInSync: boolean;
    socketPort?: number;
    tunnel: string;
    tunnelOptions?: TunnelOptions | BrowserStackOptions | SeleniumOptions;
}
export interface Remote extends Command<any> {
    environmentType?: Environment;
    setHeartbeatInterval(delay: number): Command<any>;
}
export interface EnvironmentSpec {
    browserName: string;
    [key: string]: any;
}
export interface TunnelMessage {
    tunnel: Tunnel;
    progress?: DownloadProgressEvent;
    status?: string;
}
export interface Events extends BaseEvents {
    serverEnd: Server;
    serverStart: Server;
    sessionStart: Remote;
    sessionEnd: Remote;
    tunnelDownloadProgress: TunnelMessage;
    tunnelStart: TunnelMessage;
    tunnelStatus: TunnelMessage;
    tunnelStop: TunnelMessage;
}
